#include <stdio.h>
#include <math.h>
#include "oct.h"
#include "my.h"

#define NIL(type)	((type *) 0)



#define UNITS_PER_LAMBDA	20
#define WIDTH		0

#define WIRING	"WIRING"
#define SYMBOL	"SYMBOL"


    /* Base-line height */
#define HEIGHT		(10*UNITS_PER_LAMBDA)
    /* Straight-line part for NAND gate */
#define NAND_BASE	(7*UNITS_PER_LAMBDA)
    /* Straight-line part for NOR gate */
#define NOR_BASE	(4*UNITS_PER_LAMBDA)
    /* Offset between double lines on XOR */
#define XOR_OFFSET	(1*UNITS_PER_LAMBDA)

    /* indent on circle left edge */
#define NOR_INDENT	45		
    /* indent on circles top and bottom */
#define NOR_INDENT1	32		

    /* Size of terminal */
#define TERM_SIZE	2
    /* Size of stem leading to the terminal */
#define LEADER		(1*UNITS_PER_LAMBDA)

    /* inverter bubble size */
#define INV_RADIUS	(1*UNITS_PER_LAMBDA)	

#define USE_PATH_NOT_CIRCLE	1

#define CONTACT_SIZE	(1*20)
#define CONTACT_CIRCLE	0		/* contact is a circle */

int space(n)
int n;
{
    switch(n) {
    case 2: return 4*UNITS_PER_LAMBDA;
    case 3: return 3*UNITS_PER_LAMBDA;
    case 4: return 2*UNITS_PER_LAMBDA;
    case 5: return 2*UNITS_PER_LAMBDA;
    case 6: return 1*UNITS_PER_LAMBDA;
    case 7: return 1*UNITS_PER_LAMBDA;
    case 8: return 1*UNITS_PER_LAMBDA;
    default: return 1*UNITS_PER_LAMBDA;
    }
}

main()
{
    octObject facet;
    char name[1024];
    int n;

    for(n = 2; n <= 5; n++) {
	(void) sprintf(name, "and%d", n);
	OCT_ASSERT(make_and(&facet, name, n, 0));

	(void) sprintf(name, "nand%d", n);
	OCT_ASSERT(make_and(&facet, name, n, 1));

	(void) sprintf(name, "or%d", n);
	OCT_ASSERT(make_or(&facet, name, n, /* bubble */0, /* xor */ 0));

	(void) sprintf(name, "nor%d", n);
	OCT_ASSERT(make_or(&facet, name, n, /* bubble */1, /* xor */ 0));
    }
    OCT_ASSERT(make_contact());
    OCT_ASSERT(make_inv(&facet, "inv", 1));

    OCT_ASSERT(make_or(&facet, "xor2", 2, /* bubble */0, /* xor */1));
    OCT_ASSERT(make_or(&facet, "xnor2", 2, /* bubble */1, /* xor */1));
}


/*
 *  create an 'n-input' and/nand gate schematic symbol
 */

octStatus make_and(facet, name, n, invert_output)
octObject *facet;
char *name;
int n;
int invert_output;
{
    octObject interface, prop, term, layer_symbol, layer_wiring, box, path, circle;
    octCoord base, spacing, xpos, ypos, xl;
    char name1[10];
    int i;

    /* Open the facet for over-write */
    CHECK(OpenFacet(facet, name, "schematic", "contents", "w"));

    /* Drop some properties down */
    CHECK(CreateSprop(facet, &prop, "EDITSTYLE", "PHYSICAL"));
    CHECK(CreateSprop(facet, &prop, "TECHNOLOGY", "schematic"));
    CHECK(CreateSprop(facet, &prop, "VIEWTYPE", "SCHEMATIC"));
    CHECK(CreateSprop(facet, &prop, "CELLTYPE", "COMBINATIONAL"));
    CHECK(CreateSprop(facet, &prop, "CELLCLASS", "LEAF"));

    /* Create the layers */
    CHECK(GetOrCreateLayer(facet, &layer_symbol, SYMBOL));
    CHECK(GetOrCreateLayer(facet, &layer_wiring, WIRING));

    /* Create the easy lines */
    CHECK(CreatePath2(&layer_symbol, &path, 0, 0, 0, HEIGHT, WIDTH));
    CHECK(CreatePath2(&layer_symbol, &path, 0, HEIGHT, NAND_BASE, HEIGHT, WIDTH));
    CHECK(CreatePath2(&layer_symbol, &path, 0, 0, NAND_BASE, 0, WIDTH));
    CHECK(CreateCircle(&layer_symbol, &circle, 270*10, 90*10, 
			    HEIGHT/2 - WIDTH, HEIGHT/2, NAND_BASE, HEIGHT/2));
    CHECK(hack_circle_into_path(&circle));
    xl = NAND_BASE + HEIGHT/2;


    /* Create the terminals */
    spacing = space(n);
    base = (((HEIGHT - (n-1)*spacing)/2)/20)*20;
    for(i = 0; i < n; i++) {
	
	/* Create the line leading to the terminal */
	xpos = -LEADER;
	ypos = base + i*spacing;
	if (LEADER > 0) {
	    CHECK(CreatePath2(&layer_symbol, &path, -LEADER, ypos, 0, ypos, WIDTH));
	}
	CHECK(CreateBox(&layer_wiring, &box,
	    xpos-TERM_SIZE/2, ypos-TERM_SIZE/2,
	    xpos+TERM_SIZE/2, ypos+TERM_SIZE/2));

	/* Create the input terminal */
	(void) sprintf(name1, "1%c", "ABCDEFGHIJKLMNOPQRSTUVWXYZ"[i]);
	CHECK(CreateTerm(facet, &term, name1));
	CHECK(octAttach(&term, &box));
    }

    /* Create the arc */
    if (invert_output) {
	CHECK(CreateCircle(&layer_symbol, &circle, 0, 3600,
		INV_RADIUS - WIDTH, INV_RADIUS, 	/* radii */
		xl + INV_RADIUS, HEIGHT/2));		/* center */
	xl += INV_RADIUS*2;
    }


    /* Create the output lines */
    xpos = xl;
    ypos = HEIGHT/2;
    CHECK(CreatePath2(&layer_symbol, &path, xpos, ypos, xpos+LEADER, ypos, WIDTH));
    xpos += LEADER;
    CHECK(CreateBox(&layer_wiring, &box,
	xpos-TERM_SIZE/2, ypos-TERM_SIZE/2,
	xpos+TERM_SIZE/2, ypos+TERM_SIZE/2));

    /* Create the output terminal */
    CHECK(CreateTerm(facet, &term, "O"));
    CHECK(octAttach(&term, &box));

    /* Add LOGICFUNCTION, TERMTYPE and DIRECTION properties */
    CHECK(add_properties(facet, "*", n, invert_output));

    /* Close things up */
    CHECK(octCloseFacet(facet));

    /* Copy the contents to the interface */
    interface = *facet;
    interface.contents.facet.facet = "interface";
    CHECK(octCopyFacet(&interface, facet));

    return OCT_OK;
}

octCoord octAngle(x)
double x;
{
    return (octCoord) (x * 10.0 * 360.0/(2*3.1415926535));
}


/*
 *  Draw a circle given a chord length (h), and a distance from the coord 
 *  to the circle (x).  The midpoint of the chord is placed at (xpos, ypos).
 */
octStatus MakeCircle(layer, circle, xpos, ypos, h, x)
octObject *layer, *circle;
double xpos, ypos, h, x;
{
    double r, cx, cy;
    octCoord startAng, endAng;

    r = h*h / (8.0 * x) + x / 2.0;
    cx = xpos - (r - x);
    cy = ypos;
    startAng = 3600 - octAngle(asin(h / (2.0 * r)));
    endAng = octAngle(asin(h / (2.0 * r)));
    CHECK(CreateCircle(layer, circle, startAng, endAng, 
			(octCoord) r, (octCoord) r, 
			(octCoord) cx, (octCoord) cy));
    return OCT_OK;
}



octStatus make_or(facet, name, n, invert_output, xor_flag)
octObject *facet;
char *name;
int n;
int invert_output;
int xor_flag;
{
    struct octTransform transform;
    octObject interface, prop, term, layer_wiring, layer_symbol, box;
    octObject path, circle;
    char name1[10];
    int i;
    octCoord base, spacing, cx, xpos, ypos, xwidth;
    double d, r;
    double x1, x2, y1, y2, theta;

    xwidth = NAND_BASE + HEIGHT/2;

    CHECK(OpenFacet(facet, name, "schematic", "contents", "w"));

    /* Drop the necessary properties */
    CHECK(CreateSprop(facet, &prop, "EDITSTYLE", "PHYSICAL"));
    CHECK(CreateSprop(facet, &prop, "TECHNOLOGY", "schematic"));
    CHECK(CreateSprop(facet, &prop, "VIEWTYPE", "SCHEMATIC"));
    CHECK(CreateSprop(facet, &prop, "CELLTYPE", "COMBINATIONAL"));
    CHECK(CreateSprop(facet, &prop, "CELLCLASS", "LEAF"));

    /* Create the layers */
    CHECK(CreateLayer(facet, &layer_wiring, WIRING));
    CHECK(CreateLayer(facet, &layer_symbol, SYMBOL));

    /* Draw an arc for the "inputs" */
    CHECK(MakeCircle(&layer_symbol, &circle, 0.0, HEIGHT / 2.0, 
			(double) HEIGHT, (double) NOR_INDENT));
    CHECK(hack_circle_into_path(&circle));
    r = circle.contents.circle.innerRadius;
    cx = circle.contents.circle.center.x;

    /* Draw an arc for the "inputs" */
    if (xor_flag) {
	CHECK(MakeCircle(&layer_symbol, &circle, (double) -XOR_OFFSET, HEIGHT / 2.0, 
			    (double) HEIGHT, (double) NOR_INDENT));
	CHECK(hack_circle_into_path(&circle));
	r = circle.contents.circle.innerRadius;
	cx = circle.contents.circle.center.x;
    }


    /* Draw the flat part on the top and bottom */
    CHECK(CreatePath2(&layer_symbol, &path, 0, 0, NOR_BASE, 0, WIDTH));
    CHECK(CreatePath2(&layer_symbol, &path, 0, HEIGHT, NOR_BASE, HEIGHT, WIDTH));

    /* Draw arc for the top */
    x1 = NOR_BASE;
    y1 = HEIGHT;
    x2 = xwidth;
    y2 = HEIGHT / 2.0;
    d = hypot(y2 - y1, x2 - x1);
    theta = atan2(y1 - y2, x1 - x2) - 3.141592654/2.0;
    CHECK(MakeCircle(&layer_symbol, &circle, 0.0, 0.0, d, (double) NOR_INDENT1));
    transform.transformType = OCT_FULL_TRANSFORM;
    transform.translation.x = (octCoord) (x1 + x2) / 2.0;
    transform.translation.y = (octCoord) (y1 + y2) / 2.0;
    transform.generalTransform[0][0] = cos(theta);
    transform.generalTransform[0][1] = - sin(theta);
    transform.generalTransform[1][0] = sin(theta);
    transform.generalTransform[1][1] = cos(theta);
    OCT_ASSERT(octTransformAndModifyGeo(&circle, &transform));
    CHECK(hack_circle_into_path(&circle));

    /* Draw arc for the bottom */
    x1 = xwidth;
    y1 = HEIGHT / 2.0;
    x2 = NOR_BASE;
    y2 = 0.0;
    d = hypot(y2 - y1, x2 - x1);
    theta = atan2(y1 - y2, x1 - x2) - 3.141592654/2.0;
    CHECK(MakeCircle(&layer_symbol, &circle, 0.0, 0.0, d, (double) NOR_INDENT1));
    transform.transformType = OCT_FULL_TRANSFORM;
    transform.translation.x = (octCoord) (x1 + x2) / 2.0;
    transform.translation.y = (octCoord) (y1 + y2) / 2.0;
    transform.generalTransform[0][0] = cos(theta);
    transform.generalTransform[0][1] = - sin(theta);
    transform.generalTransform[1][0] = sin(theta);
    transform.generalTransform[1][1] = cos(theta);
    OCT_ASSERT(octTransformAndModifyGeo(&circle, &transform));
    CHECK(hack_circle_into_path(&circle));


    /* Create the input terminals */
    spacing = space(n);
    base = (((HEIGHT - (n-1)*spacing)/2)/20)*20;
    for(i = 0; i < n; i++) {

	/* Draw a line from the edge to the base-line for the nor gate */
	ypos = base + i*spacing;
	xpos = cx + sqrt(r*r - (ypos - HEIGHT/2.0)*(ypos - HEIGHT/2.0));
	if (LEADER > 0) {
	    CHECK(CreatePath2(&layer_symbol, &path, -LEADER, ypos, xpos, ypos, WIDTH));
	}

	/* Create the terminal and its implementation */
	xpos = - LEADER;
	CHECK(CreateBox(&layer_wiring, &box, 
		xpos-TERM_SIZE/2, ypos-TERM_SIZE/2,
		xpos+TERM_SIZE/2, ypos+TERM_SIZE/2));

	(void) sprintf(name1, "1%c", "ABCDEFGHIJKLMNOPQRSTUVWXYZ"[i]);

	/* Create the input terminal */
	CHECK(CreateTerm(facet, &term, name1));
	CHECK(octAttach(&term, &box));
    }


    /* Create the arc */
    if (invert_output) {
	CHECK(CreateCircle(&layer_symbol, &circle, 0, 3600, 
		    INV_RADIUS - WIDTH, INV_RADIUS, 
		    xwidth + INV_RADIUS, HEIGHT/2));
	xwidth += INV_RADIUS*2;
    }


    /* Create the output line */
    xpos = xwidth;
    ypos = HEIGHT/2;
    if (LEADER > 0) {
	CHECK(CreatePath2(&layer_symbol, &path, xpos, ypos, xpos+LEADER, ypos, WIDTH));
    }
    xpos += LEADER;
    CHECK(CreateBox(&layer_wiring, &box, 
		xpos-TERM_SIZE/2, ypos-TERM_SIZE/2,
		xpos+TERM_SIZE/2, ypos+TERM_SIZE/2));

    /* Create the output terminal */
    CHECK(CreateTerm(facet, &term, "O"));
    CHECK(octAttach(&term, &box));

    /* Add LOGICFUNCTION, TERMTYPE and DIRECTION properties */
    CHECK(add_properties(facet, "+", n, invert_output));

    /* Close things up */
    CHECK(octCloseFacet(facet));

    /* Copy the contents to the interface */
    interface = *facet;
    interface.contents.facet.facet = "interface";
    CHECK(octCopyFacet(&interface, facet));

    return OCT_OK;
}

octStatus make_contact()
{
    octObject interface, layer_symbol, layer_wiring, term, prop, facet, obj;

    /* Open the facet for over-write */
    CHECK(OpenFacet(&facet, "contact", "schematic", "contents", "w"));

    /* Create the necessary properties */
    CHECK(CreateSprop(&facet, &prop, "EDITSTYLE", "PHYSICAL"));
    CHECK(CreateSprop(&facet, &prop, "TECHNOLOGY", "schematic"));
    CHECK(CreateSprop(&facet, &prop, "VIEWTYPE", "SCHEMATIC"));

    /* Create the layer for the wiring */
    CHECK(CreateLayer(&facet, &layer_symbol, SYMBOL));
    CHECK(CreateLayer(&facet, &layer_wiring, WIRING));

    if (CONTACT_CIRCLE) {
	CHECK(CreateCircle(&layer_wiring, &obj, 0, 3600, 
				0, CONTACT_SIZE/2, 0, 0));
    } else {
	CHECK(CreateBox(&layer_wiring, &obj,
		-CONTACT_SIZE/2, -CONTACT_SIZE/2,
		CONTACT_SIZE/2, CONTACT_SIZE/2));
    }

    /* Create the terminal */
    CHECK(CreateTerm(&facet, &term, "t"));
    CHECK(octAttach(&term, &obj));

    /* Add properties */
    CHECK(CreateSprop(&facet, &prop, "CELLTYPE", "CONTACT"));

    /* Close things up */
    CHECK(octCloseFacet(&facet));

    /* Copy the contents to the interface */
    interface = facet;
    interface.contents.facet.facet = "interface";
    CHECK(octCopyFacet(&interface, &facet));

    return OCT_OK;
}

make_inv(facet, name, invert_output)
octObject *facet;
char *name;
int invert_output;
{
    octObject interface, prop, term, layer_symbol, layer_wiring, box, path, circle;
    octCoord xpos, ypos, xl;

    /* Open the facet for over-write */
    CHECK(OpenFacet(facet, name,  "schematic", "contents", "w"));

    /* Create the necessary properties */
    CHECK(CreateSprop(facet, &prop, "EDITSTYLE", "PHYSICAL"));
    CHECK(CreateSprop(facet, &prop, "TECHNOLOGY", "schematic"));
    CHECK(CreateSprop(facet, &prop, "VIEWTYPE", "SCHEMATIC"));
    CHECK(CreateSprop(facet, &prop, "CELLTYPE", "COMBINATIONAL"));
    CHECK(CreateSprop(facet, &prop, "CELLCLASS", "LEAF"));

    /* Create the layer for the wiring */
    CHECK(CreateLayer(facet, &layer_wiring, WIRING));
    CHECK(CreateLayer(facet, &layer_symbol, SYMBOL));

    /* Create the easy lines */
    xl = HEIGHT;
    CHECK(CreatePath2(&layer_symbol, &path, 0, 0, 0, HEIGHT, WIDTH));
    CHECK(CreatePath2(&layer_symbol, &path, 0, HEIGHT, xl, HEIGHT/2, WIDTH));
    CHECK(CreatePath2(&layer_symbol, &path, 0, 0, xl, HEIGHT/2, WIDTH));

    /* Create the line leading to the terminal */
    xpos = -LEADER;
    ypos = HEIGHT/2;
    if (LEADER > 0) {
	CHECK(CreatePath2(&layer_symbol, &path, -LEADER, ypos, 0, ypos, WIDTH));
    }
    CHECK(CreateBox(&layer_wiring, &box,
	xpos-TERM_SIZE/2, ypos-TERM_SIZE/2,
	xpos+TERM_SIZE/2, ypos+TERM_SIZE/2));

    /* Create the input terminal */
    CHECK(CreateTerm(facet, &term, "1A"));
    CHECK(octAttach(&term, &box));

    /* Create the arc */
    if (invert_output) {
	CHECK(CreateCircle(&layer_symbol, &circle, 0, 3600,
		INV_RADIUS - WIDTH, INV_RADIUS, 	/* radii */
		xl + INV_RADIUS, HEIGHT/2));		/* center */
	xl += INV_RADIUS*2;
    }


    /* Create the output lines */
    xpos = xl;
    ypos = HEIGHT/2;
    if (LEADER > 0) {
	CHECK(CreatePath2(&layer_symbol, &path, xpos, ypos, xpos+LEADER, ypos, WIDTH));
    }
    xpos += LEADER;
    CHECK(CreateBox(&layer_wiring, &box,
	xpos-TERM_SIZE/2, ypos-TERM_SIZE/2,
	xpos+TERM_SIZE/2, ypos+TERM_SIZE/2));

    /* Create the output terminal */
    CHECK(CreateTerm(facet, &term, "O"));
    CHECK(octAttach(&term, &box));

    /* Add LOGICFUNCTION, TERMTYPE and DIRECTION properties */
    CHECK(add_properties(facet, "*", 1, invert_output));

    /* Close things up */
    CHECK(octCloseFacet(facet));

    /* Copy the contents to the interface */
    interface = *facet;
    interface.contents.facet.facet = "interface";
    CHECK(octCopyFacet(&interface, facet));

    return OCT_OK;
}


octStatus add_properties(facet, operator, n, invert_output)
octObject *facet;
char *operator;
int n;
int invert_output;
{
    int i;
    char name[10], logic_function[1024];
    octObject term, prop;

    /* Start the logic function string */
    (void) strcpy(logic_function, "");
    if (invert_output) {
	(void) strcat(logic_function, "(!");
    }
    (void) strcat(logic_function, "(");
    (void) strcat(logic_function, operator);

    /* Loop for each input terminal */
    for(i = 0; i < n; i++) {
	(void) sprintf(name, "1%c", "ABCDEFGHIJKLMNOPQRSTUVWXYZ"[i]);

	/* Attach properties to the terminal */
	CHECK(GetByTermName(facet, &term, name));
	CHECK(CreateSprop(&term, &prop, "TERMTYPE", "SIGNAL"));
	CHECK(CreateSprop(&term, &prop, "DIRECTION", "INPUT"));

	/* Append this input to the logic function */
	(void) strcat(logic_function, " ");
	(void) strcat(logic_function, name);
    }

    /* Finish off the logic_function */
    (void) strcat(logic_function, ")");
    if (invert_output) {
	(void) strcat(logic_function, ")");
    }

    /* Attach properties to the output terminal */
    CHECK(GetByTermName(facet, &term, "O"));
    CHECK(CreateSprop(&term, &prop, "LOGICFUNCTION", logic_function));
    CHECK(CreateSprop(&term, &prop, "TERMTYPE", "SIGNAL"));
    CHECK(CreateSprop(&term, &prop, "DIRECTION", "OUTPUT"));
    return OCT_OK;
}

#define NSTEP	25
	
octStatus hack_circle_into_path(circle)
octObject *circle;
{
#if USE_PATH_NOT_CIRCLE
    octObject layer, path;
    octStatus status;
    struct octPoint point[NSTEP];
    double ang, start, end, step, cx, cy, radius, thick;
    int i;

    start = circle->contents.circle.startingAngle * 2*3.141592654 / 3600.0;
    end = circle->contents.circle.endingAngle * 2*3.141592654 / 3600.0;
    if (start > end) end += 2*3.141592654;

    cx = circle->contents.circle.center.x;
    cy = circle->contents.circle.center.y;
    radius = circle->contents.circle.outerRadius;
    thick = circle->contents.circle.outerRadius - 
		    circle->contents.circle.innerRadius;

    step = (end - start) / (NSTEP - 1);

    for(i = 0; i < NSTEP; i++) {
	ang = start + step*i;
	point[i].x = cx + radius * cos(ang);
	point[i].y = cy + radius * sin(ang);
    }

    CHECK(status = myGenFirstContainer(circle, OCT_LAYER_MASK, &layer));
    if (status == OCT_GEN_DONE) return OCT_ERROR;
    CHECK(CreatePath(&layer, &path, (octCoord) thick));
    CHECK(octPutPoints(&path, NSTEP, point));
    CHECK(octDelete(circle));
#endif
    return OCT_OK;
}
