/*	MSU DELAYS
	----------

Utility to attach to the cells of the MSU standard cell set HUM-DELAY-BAGs,
containing HUM-UP-DELAY, HUM-DOWN-DELAY and HUM-DEPENDENCY properties,

Nicholas Weiner, 22nd June 1987.

*/

#include <stdio.h>
#include "oct.h"

/* GLOBALS */
char   *MSUview = "physical";


/*
* MAIN

*  Worst case delay data from MSU library document  
*  (~cad/lib/technology/scmos/msu).
*  Data for 2 micron cells using double layer metal (DLM).
*
* Changes to required values of the parameters, and additions to
* the list should be made in this main routine.
*
* The routine MSUattach has the following interface:
*     MSUattach(CellName,input_name,output_name,up_delay,down_delay,dependency)
*/

main(argc,argv)
    int argc;
    char **argv;
    {


    octBegin();


    /*      CELL-NAME INPUT OUTPUT UP-DELAY DOWN-DELAY     DEPENDENCY */

    /* 2 inverters */
    MSUattach("1100", "1A", "O1",  4.07e-9, 3.30e-9, "MONOTONIC-DECREASING");
    MSUattach("1100", "2B", "O2",  4.07e-9, 3.30e-9, "MONOTONIC-DECREASING");

    /* 2 input NOR */
    MSUattach("1120", "1A", "O", 11.33e-9, 9.81e-9, "MONOTONIC-DECREASING");
    MSUattach("1120", "1B", "O", 11.33e-9, 9.81e-9, "MONOTONIC-DECREASING");

    /* 3 input NOR */
    MSUattach("1130", "1A", "O", 17.14e-9, 12.6e-9, "MONOTONIC-DECREASING");
    MSUattach("1130", "1B", "O", 17.14e-9, 12.6e-9, "MONOTONIC-DECREASING");
    MSUattach("1130", "1C", "O", 17.14e-9, 12.6e-9, "MONOTONIC-DECREASING");

    /* 4 input NOR */
    MSUattach("1140", "1A", "O", 21e-9, 12.95e-9, "MONOTONIC-DECREASING");
    MSUattach("1140", "1B", "O", 21e-9, 12.95e-9, "MONOTONIC-DECREASING");
    MSUattach("1140", "1C", "O", 21e-9, 12.95e-9, "MONOTONIC-DECREASING");
    MSUattach("1140", "1D", "O", 21e-9, 12.95e-9, "MONOTONIC-DECREASING");

    /* 2 input NAND */
    MSUattach("1220", "1A", "O", 21.09e-9, 22.47e-9, "MONOTONIC-DECREASING");
    MSUattach("1220", "1B", "O", 21.09e-9, 22.47e-9, "MONOTONIC-DECREASING");

    /* 3 input NAND */
    MSUattach("1230", "1A", "O", 30.88e-9, 31.45e-9, "MONOTONIC-DECREASING");
    MSUattach("1230", "1B", "O", 30.88e-9, 31.45e-9, "MONOTONIC-DECREASING");
    MSUattach("1230", "1C", "O", 30.88e-9, 31.45e-9, "MONOTONIC-DECREASING");

    /* 4 input NAND - 1240 - data not found */

    /* Single inverter */
    MSUattach("1310", "1A", "O", 18.41e-9, 17.37e-9, "MONOTONIC-DECREASING");

    /* Non-inverting buffer */
    MSUattach("1320", "1A", "O", 23.38e-9, 21.75e-9, "MONOTONIC-INCREASING");

    /* Tristate buffrer */
    MSUattach("1340","1DATA", "O", 28.81e-9, 23.49e-9, "MONOTONIC-INCREASING");
    MSUattach("1340","2EN","O", 21.63-9,  25.13e-9, "MONOTONIC-INCREASING");

    /* Multiplexer */
    MSUattach("1350", "1D1", "O", 13.17e-9, 11.96e-9, "MONOTONIC-INCREASING");
    MSUattach("1350", "2D2", "O", 12.29e-9, 11.93e-9, "MONOTONIC-INCREASING");
    MSUattach("1350", "3SEL","O", 14.25e-9, 13.18e-9, "NOT-MONOTONIC");

    /* Transmission gate */
    MSUattach("1370","1DATA",  "O", 14.86e-9, 14.55e-9, "MONOTONIC-INCREASING");
    MSUattach("1370", "2CTL", "O", 9.72e-9,  7.38e-9, "NOT-MONOTONIC");

    /* 2 input nand/and */
    MSUattach("1660", "1A",  "O1", 10.78e-9, 11.04e-9, "MONOTONIC-DECREASING");
    MSUattach("1660", "1B",  "O1", 10.78e-9, 11.04e-9, "MONOTONIC-DECREASING");
    MSUattach("1660", "1A",  "O2", 9.83e-9,  8.83e-9, "MONOTONIC-INCREASING");
    MSUattach("1660", "1B",  "O2", 9.83e-9,  8.83e-9, "MONOTONIC-INCREASING");

    /* 3 input nand/and */
    MSUattach("1670", "1A",  "O1", 15.17e-9, 12.32e-9, "MONOTONIC-DECREASING");
    MSUattach("1670", "1B",  "O1", 15.17e-9, 12.32e-9, "MONOTONIC-DECREASING");
    MSUattach("1670", "1C",  "O1", 15.17e-9, 12.32e-9, "MONOTONIC-DECREASING");
    MSUattach("1670", "1A",  "O2", 11.32e-9, 16.40e-9, "MONOTONIC-INCREASING");
    MSUattach("1670", "1B",  "O2", 11.32e-9, 16.40e-9, "MONOTONIC-INCREASING");
    MSUattach("1670", "1C",  "O2", 11.32e-9, 16.40e-9, "MONOTONIC-INCREASING");

    /* 4 input nand/and */
    MSUattach("1680", "1A",  "O1", 17.49e-9, 14.88e-9, "MONOTONIC-DECREASING");
    MSUattach("1680", "1B",  "O1", 17.49e-9, 14.88e-9, "MONOTONIC-DECREASING");
    MSUattach("1680", "1C",  "O1", 17.49e-9, 14.88e-9, "MONOTONIC-DECREASING");
    MSUattach("1680", "1D",  "O1", 17.49e-9, 14.88e-9, "MONOTONIC-DECREASING");
    MSUattach("1680", "1A",  "O2", 16.76e-9, 15.14e-9, "MONOTONIC-INCREASING");
    MSUattach("1680", "1B",  "O2", 16.76e-9, 15.14e-9, "MONOTONIC-INCREASING");
    MSUattach("1680", "1C",  "O2", 16.76e-9, 15.14e-9, "MONOTONIC-INCREASING");
    MSUattach("1680", "1D",  "O2", 16.76e-9, 15.14e-9, "MONOTONIC-INCREASING");

    /* 4 input OR */
    MSUattach("1740", "1A",  "O",  9.26e-9, 17.61e-9, "MONOTONIC-INCREASING");
    MSUattach("1740", "1B",  "O",  9.26e-9, 17.61e-9, "MONOTONIC-INCREASING");
    MSUattach("1740", "1C",  "O",  9.26e-9, 17.61e-9, "MONOTONIC-INCREASING");
    MSUattach("1740", "1D",  "O",  9.26e-9, 17.61e-9, "MONOTONIC-INCREASING");

    /* 2 input OR/NOR */
    MSUattach("1760", "1A",  "O1", 9.49e-9, 9.82e-9, "MONOTONIC-INCREASING");
    MSUattach("1760", "1B",  "O1", 9.49e-9, 9.82e-9, "MONOTONIC-INCREASING");
    MSUattach("1760", "1A",  "O2", 12.55e-9,  10.05e-9, "MONOTONIC-DECREASING");
    MSUattach("1760", "1B",  "O2", 12.55e-9,  10.05e-9, "MONOTONIC-DECREASING");

    /* 3 input OR/NOR */
    MSUattach("1770", "1A",  "O1", 8.26e-9, 17.01e-9, "MONOTONIC-INCREASING");
    MSUattach("1770", "1B",  "O1", 8.26e-9, 17.01e-9, "MONOTONIC-INCREASING");
    MSUattach("1770", "1C",  "O1", 8.26e-9, 17.01e-9, "MONOTONIC-INCREASING");
    MSUattach("1770", "1A",  "O2",  20.38e-9, 10.37e-9, "MONOTONIC-DECREASING");
    MSUattach("1770", "1B",  "O2",  20.38e-9, 10.37e-9, "MONOTONIC-DECREASING");
    MSUattach("1770", "1C",  "O2",  20.38e-9, 10.37e-9, "MONOTONIC-DECREASING");

    /* AO222 ie (AB)+(CD)+(EF) */
    MSUattach("1810", "1A",  "O",  14.74e-9, 10.39e-9, "MONOTONIC-INCREASING");
    MSUattach("1810", "1B",  "O",  14.74e-9, 10.39e-9, "MONOTONIC-INCREASING");
    MSUattach("1810", "2C",  "O",  16.85e-9, 14.97e-9, "MONOTONIC-INCREASING");
    MSUattach("1810", "2D",  "O",  16.85e-9, 14.97e-9, "MONOTONIC-INCREASING");
    MSUattach("1810", "3E",  "O",  13.94e-9, 14.77e-9, "MONOTONIC-INCREASING");
    MSUattach("1810", "3F",  "O",  13.94e-9, 14.77e-9, "MONOTONIC-INCREASING");

    /* Full Adder */
    MSUattach("1850", "1A",  "SUM",  16.9e-9, 16.06e-9, "NOT-MONOTONIC");
    MSUattach("1850", "1B",  "SUM",  16.9e-9, 16.06e-9, "NOT-MONOTONIC");
    MSUattach("1850", "2CIN", "SUM", 14.20e-9, 16.51e-9, "NOT-MONOTONIC");
    MSUattach("1850", "1A", "CO",  15.0e-9, 13.97e-9, "NOT-MONOTONIC");
    MSUattach("1850", "1B", "CO",  15.0e-9, 13.97e-9, "NOT-MONOTONIC");
    MSUattach("1850", "2CIN", "CO",  10.23e-9,  16.12e-9, "NOT-MONOTONIC");

    /* AOI22 ie !( (AB)+(CD) )*/
    MSUattach("1870", "1A",  "O",  14.77e-9, 11.20e-9, "MONOTONIC-DECREASING");
    MSUattach("1870", "1B",  "O",  14.77e-9, 11.20e-9, "MONOTONIC-DECREASING");
    MSUattach("1870", "2C",  "O",  14.77e-9, 15.85e-9, "MONOTONIC-DECREASING");
    MSUattach("1870", "2D",  "O",  14.77e-9, 15.85e-9, "MONOTONIC-DECREASING");

    /* AO2222 ie ( (AB)+(CD)+(EF)+(GH) )*/
    MSUattach("1910", "1A",  "O",  15.20e-9, 16.05e-9, "MONOTONIC-INCREASING");
    MSUattach("1910", "1B",  "O",  15.20e-9, 16.05e-9, "MONOTONIC-INCREASING");
    MSUattach("1910", "2C",  "O",  15.38e-9, 15.35e-9, "MONOTONIC-INCREASING");
    MSUattach("1910", "2D",  "O",  15.38e-9, 15.35e-9, "MONOTONIC-INCREASING");
    MSUattach("1910", "3E",  "O",  13.94e-9, 16.00e-9, "MONOTONIC-INCREASING");
    MSUattach("1910", "3F",  "O",  13.94e-9, 16.00e-9, "MONOTONIC-INCREASING");
    MSUattach("1910", "4G",  "O",  15.72e-9, 15.87e-9, "MONOTONIC-INCREASING");
    MSUattach("1910", "4H",  "O",  15.72e-9, 15.87e-9, "MONOTONIC-INCREASING");

    /* AO33 ie ( (ABC)+(DEF) )*/
    MSUattach("1930", "1A",  "O",  16.79e-9, 9.14e-9, "MONOTONIC-INCREASING");
    MSUattach("1930", "1B",  "O",  16.79e-9, 9.14e-9, "MONOTONIC-INCREASING");
    MSUattach("1930", "1C",  "O",  16.79e-9, 9.14e-9, "MONOTONIC-INCREASING");
    MSUattach("1930", "2D",  "O",  16.10e-9,  14.00e-9, "MONOTONIC-INCREASING");
    MSUattach("1930", "2E",  "O",  16.10e-9,  14.00e-9, "MONOTONIC-INCREASING");
    MSUattach("1930", "2F",  "O",  16.10e-9,  14.00e-9, "MONOTONIC-INCREASING");

    /* AO22 ie ( (AB)+(CD) )*/
    MSUattach("1970", "1A",  "O", 7.58e-9, 16.09e-9, "MONOTONIC-INCREASING");
    MSUattach("1970", "1B",  "O", 7.58e-9, 16.09e-9, "MONOTONIC-INCREASING");
    MSUattach("1970", "2C",  "O", 7.58e-9, 16.09e-9, "MONOTONIC-INCREASING");
    MSUattach("1970", "2D",  "O", 7.58e-9, 16.09e-9, "MONOTONIC-INCREASING");

    /* XOR */
    MSUattach("2310", "1A",  "O", 17.07e-9, 16.82e-9, "NOT-MONOTONIC");
    MSUattach("2310", "1B",  "O", 17.07e-9, 16.82e-9, "NOT-MONOTONIC");

    /* XNOR  2350 no delay data - Used same values as for 2310 */
    MSUattach("2350", "1A",  "O", 17.07e-9, 16.82e-9, "NOT-MONOTONIC");
    MSUattach("2350", "1B",  "O", 17.07e-9, 16.82e-9, "NOT-MONOTONIC");

    /* A + !B */
    MSUattach("1620", "1A", "O",  9.78e-9,  10.54e-9, "MONOTONIC-INCREASING");
    MSUattach("1620", "2B", "O",  9.51e-9,  10.54e-9, "MONOTONIC-DECREASING");

    /* (!A).B */
    MSUattach("1610", "1A", "O", 11.39e-9, 8.92e-9, "MONOTONIC-DECREASING");
    MSUattach("1610", "2B", "O", 11.39e-9, 8.16e-9, "MONOTONIC-INCREASING");

    /* 4x (big) inverting buffer */
    MSUattach("1520", "1A", "O",  7.79e-9, 6.60e-9, "MONOTONIC-DECREASING");

    /* 4x (big) inverting buffer */
    MSUattach("1520", "1A", "O",  7.79e-9, 6.60e-9, "MONOTONIC-DECREASING");

    /* aoi21 ie !(a + bc). Delay data not available - invented values */
    MSUattach("1880", "1A", "O", 14.77e-9,  11.20e-9, "MONOTONIC-DECREASING");
    MSUattach("1880", "2B", "O", 14.77e-9,  15.85e-9, "MONOTONIC-DECREASING");
    MSUattach("1880", "2C", "O", 14.77e-9,  15.85e-9, "MONOTONIC-DECREASING");

    /* oai22 ie !( (a+b).(c+d) ). Delay data not available - invented values */
    MSUattach("1860", "1A", "O", 30.00e-9,  30.00e-9, "MONOTONIC-DECREASING");
    MSUattach("1860", "1B", "O", 30.00e-9,  30.00e-9, "MONOTONIC-DECREASING");
    MSUattach("1860", "2C", "O", 30.00e-9,  30.00e-9, "MONOTONIC-DECREASING");
    MSUattach("1860", "2D", "O", 30.00e-9,  30.00e-9, "MONOTONIC-DECREASING");
    
    /* oai21 ie !( a.(b+c) ). Delay data not available - invented values */
    MSUattach("1890", "1A", "O", 30.00e-9,  30.00e-9, "MONOTONIC-DECREASING");
    MSUattach("1890", "2B", "O", 30.00e-9,  30.00e-9, "MONOTONIC-DECREASING");
    MSUattach("1890", "2C", "O", 30.00e-9,  30.00e-9, "MONOTONIC-DECREASING");
    
    /* 4 input NAND. Delay data not available - these are invented values */
    MSUattach("1240", "1A", "O", 16.00e-9,  15.00e-9, "MONOTONIC-DECREASING");
    MSUattach("1240", "1B", "O", 16.00e-9,  15.00e-9, "MONOTONIC-DECREASING");
    MSUattach("1240", "1C", "O", 16.00e-9,  15.00e-9, "MONOTONIC-DECREASING");
    MSUattach("1240", "1D", "O", 16.00e-9,  15.00e-9, "MONOTONIC-DECREASING");
    }





/*
*  MSU ATTACH
*
*  Attaches properties to interface of given cell.
*/

MSUattach(cell_name,input_name,output_name,up_delay,down_delay,dependency)
char		  *cell_name;
char		  *input_name;
char		  *output_name;
float		  up_delay;
float		  down_delay;
char		  *dependency;


    {
    /* oct variables */
    octObject CellInterface, Prop, DelayBag, Terminal, UpDelayProp, 
	      DownDelayProp, DependencyProp;

    octStatus OctReturn;


    /* Open the cell interface */
    CellInterface.type = OCT_FACET;
    CellInterface.contents.facet.cell = cell_name;
    CellInterface.contents.facet.view = MSUview;
    CellInterface.contents.facet.version = OCT_CURRENT_VERSION;
    CellInterface.contents.facet.facet = "interface";
    CellInterface.contents.facet.mode = "a";

    if(octOpenFacet(&CellInterface) < OCT_OK ) 
    	{
	octError(cell_name);
 	return;
        }

    /* Make the OCT terminal template */
    Terminal.type = OCT_TERM;

    /* Make the OCT HUM-DELAY bag template */
    DelayBag.type = OCT_BAG;
    DelayBag.contents.bag.name = "HUM-DELAY-BAG";

    /* Make the OCT Delay property templates */
    UpDelayProp.type = OCT_PROP;
    DownDelayProp.type = OCT_PROP;
    DependencyProp.type = OCT_PROP;
    UpDelayProp.contents.prop.name = "HUM-UP-DELAY";
    DownDelayProp.contents.prop.name = "HUM-DOWN-DELAY";
    DependencyProp.contents.prop.name = "HUM-DEPENDENCY";
    UpDelayProp.contents.prop.type = OCT_REAL;
    DownDelayProp.contents.prop.type = OCT_REAL;
    DependencyProp.contents.prop.type = OCT_STRING;

    /* Make HUM-DELAY bag (attach it to the input terminal) */
    /* Get the OCT instance input terminal */
    Terminal.contents.term.name = input_name;
    if( (octGetByName(&CellInterface,&Terminal)) != OCT_OK)
        {
	octError("Getting input terminal");
        printf("Cell %s, terminal %s\n",cell_name,input_name);
	return;
	}
	
    if(octCreate(&Terminal,&DelayBag) != OCT_OK)
	{
	octError("Making HUM-DELAY-BAG");
	return;
	}
       	
    /* Attach delay bag to output terminal */
    /* Get the OCT instance input terminal */
    Terminal.contents.term.name = output_name;
    if( (octGetByName(&CellInterface,&Terminal)) != OCT_OK)
        {
	octError("Getting output terminal");
        printf("Cell %s, terminal %s\n",cell_name,output_name);
	return;
	}
	
    if(octAttach(&Terminal,&DelayBag) != OCT_OK)
	{
	octError("Attaching delay to output terminal");
	return;
	}
   	
    /* Attach HUM-UP-DELAY and HUM-DOWN-DELAY properties. */
    UpDelayProp.contents.prop.value.real = (double)up_delay;
    if(octCreate(&DelayBag,&UpDelayProp) != OCT_OK)
	{
	octError("Making HUM-UP-DELAY property");
	return;
	}

    DownDelayProp.contents.prop.value.real = (double)down_delay;
    if(octCreate(&DelayBag,&DownDelayProp) != OCT_OK)
	{
	octError("Making HUM-DOWN-DELAY property");
	return;
	}

    if(strcmp(dependency,"NOT-MONOTONIC") != 0)
	{
        DependencyProp.contents.prop.value.string = dependency;
        if(octCreate(&DelayBag,&DependencyProp) != OCT_OK)
	    {
	    octError("Making HUM-DEPENDENCY property");
	    return;
	    }
   	}
    if(octFlushFacet(&CellInterface) != OCT_OK ) 
    	{
	octError(cell_name);
 	return;
        }

    return;
    }


