/* This program generates the bdnet file to
/*	UCB DELAYS
	----------

Utility to attach to the cells of the UCB standard cell set 
HUM-DELAY-BAGs, containing the properties : HUM- INTRINSIC-RDELAY;
HUM-NMOS-RDELAY-COEFFICIENT; HUM-PMOS-RDELAY-COEFFICIENT and 
HUM-LCAP-RDELAY-COEFFICIENT for rising output transitions, and  
HUM-INTRINSIC-FDELAY; HUM-NMOS-FDELAY-COEFFICIENT; 
HUM-PMOS-FDELAY-COEFFICIENT and HUM-LCAP-FDELAY-COEFFICIENT for 
fall transitions.  The bag may also contain the OCT_STRING property:
HUM-DEPENDENCY.  and the properties to the input terminals 
HUM-NMOS-AREA, HUM-PMOS-AREA.

Kanwar Jit Singh Aug 20, 1987.

*/

#include <stdio.h>
#include "oct.h"

/* GLOBALS */
char   *UCBview = "physical";


/*
* MAIN

* The routine UCBattach has the following interface:
*     UCBattach(CellName,input_name,output_name,dependency)

* The delay models are stored in the directory 
* "~cad/lib/technology/scmos/msu/dlm/delay_model".
*/

main(argc,argv)
int argc;
char **argv;
{


	octBegin();


	/*      CELL-NAME INPUT OUTPUT UP-DELAY DOWN-DELAY     DEPENDENCY */

	/* 2 inverters */
	UCBattach("1100", "1A", "O1", "MONOTONIC-DECREASING");
	UCBattach("1100", "2B", "O2", "MONOTONIC-DECREASING");

	/* 2 input NOR */
	UCBattach("1120", "1A", "O", "MONOTONIC-DECREASING");
	UCBattach("1120", "1B", "O", "MONOTONIC-DECREASING");

	/* 3 input NOR */
	UCBattach("1130", "1A", "O", "MONOTONIC-DECREASING");
	UCBattach("1130", "1B", "O", "MONOTONIC-DECREASING");
	UCBattach("1130", "1C", "O", "MONOTONIC-DECREASING");

	/* 4 input NOR */
	UCBattach("1140", "1A", "O", "MONOTONIC-DECREASING");
	UCBattach("1140", "1B", "O", "MONOTONIC-DECREASING");
	UCBattach("1140", "1C", "O", "MONOTONIC-DECREASING");
	UCBattach("1140", "1D", "O", "MONOTONIC-DECREASING");

	/* 2 input NAND */
	UCBattach("1220", "1A", "O", "MONOTONIC-DECREASING");
	UCBattach("1220", "1B", "O", "MONOTONIC-DECREASING");

	/* 3 input NAND */
	UCBattach("1230", "1A", "O", "MONOTONIC-DECREASING");
	UCBattach("1230", "1B", "O", "MONOTONIC-DECREASING");
	UCBattach("1230", "1C", "O", "MONOTONIC-DECREASING");

	/* 4 input NAND - 1240 - data not found */

	/* Single inverter */
	UCBattach("1310", "1A", "O", "MONOTONIC-DECREASING");

	/* Non-inverting buffer */
	/* No UCB model */
	/*UCBattach("1320", "1A", "O", "MONOTONIC-INCREASING");*/

	/* Tristate buffrer */
/*	UCBattach("1340","1DATA", "O", "MONOTONIC-INCREASING");
	UCBattach("1340","2EN","O", "MONOTONIC-INCREASING");*/

	/* Multiplexer */
	/*UCBattach("1350", "1D1", "O", "MONOTONIC-INCREASING");
	UCBattach("1350", "3SEL","O", "NOT-MONOTONIC"); */

	/* Transmission gate */
	/*UCBattach("1370","1DATA",  "O", "MONOTONIC-INCREASING");
	UCBattach("1370", "2CTL", "O", "NOT-MONOTONIC");*/

	/* 2 input nand/and */
	UCBattach("1660", "1A",  "O1", "MONOTONIC-DECREASING");
	UCBattach("1660", "1B",  "O1", "MONOTONIC-DECREASING");
	UCBattach("1660", "1A",  "O2", "MONOTONIC-INCREASING");
	UCBattach("1660", "1B",  "O2", "MONOTONIC-INCREASING");

	/* 3 input nand/and */
	UCBattach("1670", "1A",  "O1", "MONOTONIC-DECREASING");
	UCBattach("1670", "1B",  "O1", "MONOTONIC-DECREASING");
	UCBattach("1670", "1C",  "O1", "MONOTONIC-DECREASING");
	UCBattach("1670", "1A",  "O2", "MONOTONIC-INCREASING");
	UCBattach("1670", "1B",  "O2", "MONOTONIC-INCREASING");
	UCBattach("1670", "1C",  "O2", "MONOTONIC-INCREASING");

	/* 4 input nand/and */
	UCBattach("1680", "1A",  "O1", "MONOTONIC-DECREASING");
	UCBattach("1680", "1B",  "O1", "MONOTONIC-DECREASING");
	UCBattach("1680", "1C",  "O1", "MONOTONIC-DECREASING");
	UCBattach("1680", "1D",  "O1", "MONOTONIC-DECREASING");
	UCBattach("1680", "1A",  "O2", "MONOTONIC-INCREASING");
	UCBattach("1680", "1B",  "O2", "MONOTONIC-INCREASING");
	UCBattach("1680", "1C",  "O2", "MONOTONIC-INCREASING");
	UCBattach("1680", "1D",  "O2", "MONOTONIC-INCREASING");

	/* 4 input OR */
	UCBattach("1740", "1A",  "O", "MONOTONIC-INCREASING");
	UCBattach("1740", "1B",  "O", "MONOTONIC-INCREASING");
	UCBattach("1740", "1C",  "O", "MONOTONIC-INCREASING");
	UCBattach("1740", "1D",  "O", "MONOTONIC-INCREASING");

	/* 2 input OR/NOR */
	UCBattach("1760", "1A",  "O1", "MONOTONIC-INCREASING");
	UCBattach("1760", "1B",  "O1", "MONOTONIC-INCREASING");
	UCBattach("1760", "1A",  "O2", "MONOTONIC-DECREASING");
	UCBattach("1760", "1B",  "O2", "MONOTONIC-DECREASING");

	/* 3 input OR/NOR */
	UCBattach("1770", "1A",  "O1", "MONOTONIC-INCREASING");
	UCBattach("1770", "1B",  "O1", "MONOTONIC-INCREASING");
	UCBattach("1770", "1C",  "O1", "MONOTONIC-INCREASING");
	UCBattach("1770", "1A",  "O2", "MONOTONIC-DECREASING");
	UCBattach("1770", "1B",  "O2", "MONOTONIC-DECREASING");
	UCBattach("1770", "1C",  "O2", "MONOTONIC-DECREASING");

	/* AO222 ie (AB)+(CD)+(EF) */
	UCBattach("1810", "1A",  "O", "MONOTONIC-INCREASING");
	UCBattach("1810", "1B",  "O", "MONOTONIC-INCREASING");
	UCBattach("1810", "2C",  "O", "MONOTONIC-INCREASING");
	UCBattach("1810", "2D",  "O", "MONOTONIC-INCREASING");
	UCBattach("1810", "3E",  "O", "MONOTONIC-INCREASING");
	UCBattach("1810", "3F",  "O", "MONOTONIC-INCREASING");

	/* Full Adder */
	/*UCBattach("1850", "1A",  "SUM", "NOT-MONOTONIC");
	UCBattach("1850", "1B",  "SUM", "NOT-MONOTONIC");
	UCBattach("1850", "2CIN", "SUM, "NOT-MONOTONIC");
	    UCBattach("1850", "1A", "CO", "NOT-MONOTONIC");
	    UCBattach("1850", "1B", "CO", "NOT-MONOTONIC");
	    UCBattach("1850", "2CIN", "CO", "NOT-MONOTONIC");*/
	
	    /* AOI22 ie !( (AB)+(CD) )*/
	    UCBattach("1870", "1A",  "O", "MONOTONIC-DECREASING");
	    UCBattach("1870", "1B",  "O", "MONOTONIC-DECREASING");
	    UCBattach("1870", "2C",  "O", "MONOTONIC-DECREASING");
	    UCBattach("1870", "2D",  "O", "MONOTONIC-DECREASING");
	
	    /* AO2222 ie ( (AB)+(CD)+(EF)+(GH) )*/
	    UCBattach("1910", "1A",  "O", "MONOTONIC-INCREASING");
	    UCBattach("1910", "1B",  "O", "MONOTONIC-INCREASING");
	    UCBattach("1910", "2C",  "O", "MONOTONIC-INCREASING");
	    UCBattach("1910", "2D",  "O", "MONOTONIC-INCREASING");
	    UCBattach("1910", "3E",  "O", "MONOTONIC-INCREASING");
	    UCBattach("1910", "3F",  "O", "MONOTONIC-INCREASING");
	    UCBattach("1910", "4G",  "O", "MONOTONIC-INCREASING");
	    UCBattach("1910", "4H",  "O", "MONOTONIC-INCREASING");
	
	    /* AO33 ie ( (ABC)+(DEF) )*/
	    UCBattach("1930", "1A",  "O", "MONOTONIC-INCREASING");
	    UCBattach("1930", "1B",  "O", "MONOTONIC-INCREASING");
	    UCBattach("1930", "1C",  "O", "MONOTONIC-INCREASING");
	    UCBattach("1930", "2D",  "O", "MONOTONIC-INCREASING");
	    UCBattach("1930", "2E",  "O", "MONOTONIC-INCREASING");
	    UCBattach("1930", "2F",  "O", "MONOTONIC-INCREASING");
	
	    /* AO22 ie ( (AB)+(CD) )*/
	    UCBattach("1970", "1A",  "O", "MONOTONIC-INCREASING");
	    UCBattach("1970", "1B",  "O", "MONOTONIC-INCREASING");
	    UCBattach("1970", "2C",  "O", "MONOTONIC-INCREASING");
	    UCBattach("1970", "2D",  "O", "MONOTONIC-INCREASING");
	
	    /* XOR */
/*	    UCBattach("2310", "1A",  "O", "NOT-MONOTONIC");
	    UCBattach("2310", "1B",  "O", "NOT-MONOTONIC");*/
	
	    /* XNOR  2350 no delay data - Used same values as for 2310 */
	    /*UCBattach("2350", "1A",  "O", "NOT-MONOTONIC");
	    UCBattach("2350", "1B",  "O", "NOT-MONOTONIC");*/

	    /* A + !B */
	    UCBattach("1620", "1A", "O", "MONOTONIC-INCREASING");
	    UCBattach("1620", "2B", "O", "MONOTONIC-DECREASING");
	
	    /* (!A).B */
	    UCBattach("1610", "1A", "O", "MONOTONIC-DECREASING");
	    UCBattach("1610", "2B", "O", "MONOTONIC-INCREASING");
	
	    /* 4x (big) inverting buffer */
	    UCBattach("1520", "1A", "O", "MONOTONIC-DECREASING");
	
	    /* aoi21 ie !(a + bc). Delay data not available - invented values */
	    UCBattach("1880", "1A", "O", "MONOTONIC-DECREASING");
	    UCBattach("1880", "2B", "O", "MONOTONIC-DECREASING");
	    UCBattach("1880", "2C", "O", "MONOTONIC-DECREASING");
	
	    /* oai22 ie !( (a+b).(c+d) ). Delay data not available - invented values */
	    UCBattach("1860", "1A", "O", "MONOTONIC-DECREASING");
	    UCBattach("1860", "1B", "O", "MONOTONIC-DECREASING");
	    UCBattach("1860", "2C", "O", "MONOTONIC-DECREASING");
	    UCBattach("1860", "2D", "O", "MONOTONIC-DECREASING");
	    
	    /* oai21 ie !( a.(b+c) ). Delay data not available - invented values */
	    UCBattach("1890", "1A", "O", "MONOTONIC-DECREASING");
	    UCBattach("1890", "2B", "O", "MONOTONIC-DECREASING");
	    UCBattach("1890", "2C", "O", "MONOTONIC-DECREASING");
	    
	    /* 4 input NAND. Delay data not available - these are invented values */
	    UCBattach("1240", "1A", "O", "MONOTONIC-DECREASING");
	    UCBattach("1240", "1B", "O", "MONOTONIC-DECREASING");
	    UCBattach("1240", "1C", "O", "MONOTONIC-DECREASING");
	    UCBattach("1240", "1D", "O", "MONOTONIC-DECREASING");
}





/*
	*  UCB ATTACH
	*
	*  Attaches properties to interface of given cell.
	*/

UCBattach(cell_name,input_name,output_name,dependency)
char		  *cell_name;
char		  *input_name;
char		  *output_name;
char		  *dependency;
{
	/* oct variables */
	octObject outTerm, CellInterface, DelayBag, Terminal, DependencyProp;
	octStatus OctReturn;
	octObject Narea,Parea;
	octObject UpConst, UpC, UpN, UpP;
	octObject DownConst, DownC, DownN, DownP;
	octGenerator genTerm, genBag;
	int found_bag = 0;

	static char old_cell[20];
	FILE *input_file, *up_delay_file, *term_file, *down_delay_file;
	FILE *fopen();
	extern fclose();
	struct terminal{
		char name[20];
		double n_area;
		double p_area;
		double constup;
		double n_coeffup ;
		double p_coeffup ;
		double c_coeffup ;
		double constdown ;
		double n_coeffdown ;
		double p_coeffdown ;
		double c_coeffdown ;
		int  dependency;
	}; 

	struct terminal in_term[10];
	struct terminal out_term[4];
	char   depend[20];
	char copy[80];
	int dir, i,j;
	int inno, outno;

#define SYMMETRIC 1
#define ANTI_SYMMETRIC 2
#define BOTH 3

	printf("Cell is %s  -- Treminals %s %s\n",cell_name, input_name, output_name);
	sprintf( copy,"/cad/lib/technology/scmos/msu/dlm/delay-stuff/spice_delay/%s.input.UP", cell_name);
	if((input_file = fopen(copy,"r")) == NULL){
		printf("Unable to open input file %s \n",copy);
		return;
	}
	sprintf( copy,"/cad/lib/technology/scmos/msu/dlm/delay-stuff/spice_delay/%s.terminals", cell_name);
	if((term_file = fopen(copy,"r")) == NULL){
		printf("Unable to open terminal file %s \n",copy);
		return;
	}

	/* get the dependency info and the fact
			that there may be multiple outputs */

	i =0;
	while (fscanf(term_file,"%s %f %f \n",in_term[i].name,&(in_term[i].n_area) ,&(in_term[i].p_area) ) != EOF){
		i++;
	/*	printf ( "Input terminal %d is %s \n",i,in_term[i-1].name); */
	}

	sprintf( copy,"/cad/lib/technology/scmos/msu/dlm/delay-stuff/delay_model/%s.model.UP", cell_name);
	if((up_delay_file = fopen(copy,"r")) == NULL){
		printf("Unable to open up_delay file %s \n",copy);
		return;
	}

	sprintf( copy,"/cad/lib/technology/scmos/msu/dlm/delay-stuff/delay_model/%s.model.DOWN", cell_name);
	if((down_delay_file = fopen(copy,"r")) == NULL){
		printf("Unable to open down_delay file %s \n",copy);
		return;
	}

	fscanf(input_file,"%*c %s \n",depend);
	if ( strcmp(depend,"SYMMETRIC") == 0){
		dir = SYMMETRIC;
	}
	else if ( strcmp(depend,"ANTI_SYMMETRIC") == 0){
		dir = ANTI_SYMMETRIC;
	}
	else if ( strcmp(depend,"BOTH") == 0){
		dir = BOTH;
	}
	else{
		printf("Unknown dependency in input file \n");
		exit(-1);
	}

	/* printf(" Dependency is %s \n",depend); */
	i = 0;
	while (fscanf(up_delay_file,"%*s %*s %*s %*s %s \n",out_term[i].name) != EOF){
		if( dir == BOTH){
			if( i%2 == 0){
				fscanf(up_delay_file," %lf %lf %lf %lf \n",&(out_term[i].constdown) ,&(out_term[i].n_coeffdown) ,&(out_term[i].p_coeffdown),&(out_term[i].c_coeffdown) );
				fscanf(down_delay_file,"%*s %*s %*s %*s %*s \n"); 
				fscanf(down_delay_file," %lf %lf %lf %lf \n",&(out_term[i].constup) ,&(out_term[i].n_coeffup) ,&(out_term[i].p_coeffup),&(out_term[i].c_coeffup) );
			}
			else{
				fscanf(down_delay_file,"%*s %*s %*s %*s %*s \n"); 
				fscanf(down_delay_file," %lf %lf %lf %lf \n",&(out_term[i].constdown) ,&(out_term[i].n_coeffdown) ,&(out_term[i].p_coeffdown),&(out_term[i].c_coeffdown) );
				fscanf(up_delay_file," %lf %lf %lf %lf \n",&(out_term[i].constup) ,&(out_term[i].n_coeffup) ,&(out_term[i].p_coeffup),&(out_term[i].c_coeffup) ) ;
			}
		}
		else{
			fscanf(down_delay_file,"%*s %*s %*s %*s %*s \n"); 
			fscanf(down_delay_file," %lf %lf %lf %lf \n",&(out_term[i].constdown) ,&(out_term[i].n_coeffdown) ,&(out_term[i].p_coeffdown),&(out_term[i].c_coeffdown) );
			fscanf(up_delay_file," %lf %lf %lf %lf \n",&(out_term[i].constup) ,&(out_term[i].n_coeffup) ,&(out_term[i].p_coeffup),&(out_term[i].c_coeffup) ) ;
		}
		i++;
	}

	/* Open the cell interface */
	CellInterface.type = OCT_FACET;
	CellInterface.contents.facet.cell = cell_name;
	CellInterface.contents.facet.view = UCBview;
	CellInterface.contents.facet.version = OCT_CURRENT_VERSION;
	CellInterface.contents.facet.facet = "interface";
	CellInterface.contents.facet.mode = "a";

	if(octOpenFacet(&CellInterface) < OCT_OK ) 
	{
		octError(cell_name);
		return;
	}

	/* Make the OCT terminal template */
	Terminal.type = OCT_TERM;
	outTerm.type = OCT_TERM;

	/* Make the OCT HUM-DELAY bag template */
	DelayBag.type = OCT_BAG;

	/* Make the OCT Delay property templates */
	DownConst.type = OCT_PROP;
	DownC.type = OCT_PROP;
	DownP.type = OCT_PROP;
	DownN.type = OCT_PROP;
	UpConst.type = OCT_PROP;
	UpC.type = OCT_PROP;
	UpP.type = OCT_PROP;
	UpN.type = OCT_PROP;
	DownConst.contents.prop.name = "HUM-INTRINSIC-FDELAY";
	DownC.contents.prop.name = "HUM-LCAP-FDELAY-COEFFICIENT";
	DownN.contents.prop.name = "HUM-PMOS-FDELAY-COEFFICIENT";
	DownP.contents.prop.name = "HUM-NMOS-FDELAY-COEFFICIENT";
	UpConst.contents.prop.name = "HUM-INTRINSIC-RDELAY";
	UpC.contents.prop.name = "HUM-LCAP-RDELAY-COEFFICIENT";
	UpN.contents.prop.name = "HUM-PMOS-RDELAY-COEFFICIENT";
	UpP.contents.prop.name = "HUM-NMOS-RDELAY-COEFFICIENT";
	DownConst.contents.prop.type = OCT_REAL;
	DownC.contents.prop.type = OCT_REAL;
	DownP.contents.prop.type = OCT_REAL;
	DownN.contents.prop.type = OCT_REAL;
	UpConst.contents.prop.type = OCT_REAL;
	UpC.contents.prop.type = OCT_REAL;
	UpP.contents.prop.type = OCT_REAL;
	UpN.contents.prop.type = OCT_REAL;
	DependencyProp.type = OCT_PROP;
	DependencyProp.contents.prop.name = "HUM-DEPENDENCY";
	DependencyProp.contents.prop.type = OCT_STRING;

	/* Get the position of the current terminals fron
			the array of input and output terminals */
	inno = 0;
	while( strcmp(in_term[inno].name,input_name) != 0)
		inno++;
	outno = 0;
	while( strcmp(out_term[outno].name,output_name) != 0)
		outno++;

	/*printf("Current terminals are in %d out %d \n",inno,outno); */
	/* Make HUM-DELAY bag (attach it to the input terminal) */
	/* Get the OCT instance input terminal */
	Terminal.contents.term.name = input_name;
	if( (octGetByName(&CellInterface,&Terminal)) != OCT_OK)
	{
		printf("Cell %s, terminal %s\n",cell_name,input_name);
		return;
	}


	if(octInitGenContents(&Terminal,OCT_BAG_MASK,&genBag) != OCT_OK){
		octError("Initialising generator");
		return;
	}
	while(octGenerate(&genBag,&DelayBag) == OCT_OK){
		if( strcmp(DelayBag.contents.bag.name,"HUM-DELAY-BAG") == 0){
			if(octInitGenContainers(&DelayBag,OCT_TERM_MASK,&genTerm)!= OCT_OK){
				octError("Initialising terminal generator");
				return;
			}
			while( octGenerate(&genTerm,&outTerm) == OCT_OK){
			if(strcmp(outTerm.contents.term.name,output_name) == 0){
				found_bag = 1;
				break;
			}
			}
		}
		if( found_bag == 1) break;
	}

	if( found_bag == 0){
			/* create the bag and attach to the terminals */

			outTerm.contents.term.name = output_name;
			if( octGetByName(&CellInterface,&outTerm) != OCT_OK){
				octError("Finding the output term on interface");
				return;
			}
			DelayBag.contents.bag.name = "HUM-DELAY-BAG";
			if( octCreate(&Terminal,&DelayBag) != OCT_OK){
				octError("Creating Bag");
				return;
			}
			if( octAttach(&outTerm,&DelayBag) != OCT_OK){
				octError("attaching Bag");
				return;
			}
	}
	/* Attach the Area properties to the input
			terminal */

	Parea.type = OCT_PROP;
	Parea.contents.prop.type = OCT_REAL;
	Parea.contents.prop.name = "HUM-PMOS-AREA";
	Parea.contents.prop.value.real = (double)in_term[inno].p_area;
	if( octCreateOrModify(&Terminal,&Parea) != OCT_OK){
		octError("Creating the property HUM-PMOS-AREA");
		return;
	}

	Narea.type = OCT_PROP;
	Narea.contents.prop.type = OCT_REAL;
	Narea.contents.prop.name = "HUM-NMOS-AREA";
	Narea.contents.prop.value.real = (double)in_term[inno].n_area;
	if( octCreateOrModify(&Terminal,&Narea) != OCT_OK){
		octError("Creating the property HUM-NMOS-AREA");
		return;
	}

	UpConst.contents.prop.value.real = (double)out_term[outno].constup;
	if(octCreateOrModify(&DelayBag,&UpConst) != OCT_OK)
	{
		octError("Making HUM-INTRINSIC-RDELAY property");
		return;
	}

	UpC.contents.prop.value.real = out_term[outno].c_coeffup;
	if(octCreateOrModify(&DelayBag,&UpC) != OCT_OK)
	{
		octError("Making HUM-LCAP-RDELAY-COEFFICIENT property");
		return;
	}

	UpN.contents.prop.value.real = (double)out_term[outno].n_coeffup;
	if(octCreateOrModify(&DelayBag,&UpN) != OCT_OK)
	{
		octError("Making HUM-NMOS-RDELAY-COEFFICIENT property");
		return;
	}

	UpP.contents.prop.value.real = (double)out_term[outno].p_coeffup;
	if(octCreateOrModify(&DelayBag,&UpP) != OCT_OK)
	{
		octError("Making HUM-PMOS-RDELAY-COEFFICIENT property");
		return;
	}

	DownConst.contents.prop.value.real = (double)out_term[outno].constdown;
	if(octCreateOrModify(&DelayBag,&DownConst) != OCT_OK)
	{
		octError("Making HUM-INTRINSIC-FDELAY property");
		return;
	}

	DownN.contents.prop.value.real = (double)out_term[outno].n_coeffdown;
	if(octCreateOrModify(&DelayBag,&DownN) != OCT_OK)
	{
		octError("Making HUM-NMOS-FDELAY-COEFFICIENT property");
		return;
	}

	DownP.contents.prop.value.real = (double)out_term[outno].p_coeffdown;
	if(octCreateOrModify(&DelayBag,&DownP) != OCT_OK)
	{
		octError("Making HUM-PMOS-FDELAY-COEFFICIENT property");
		return;
	}

	DownC.contents.prop.value.real = (double)out_term[outno].c_coeffdown;
	if(octCreateOrModify(&DelayBag,&DownC) != OCT_OK)
	{
		octError("Making HUM-LCAP-FDELAY-COEFFICIENT property");
		return;
	}

	if(strcmp(dependency,"NOT-MONOTONIC") != 0)
	{
		DependencyProp.contents.prop.value.string = dependency;
		if(octCreateOrModify(&DelayBag,&DependencyProp) != OCT_OK)
		{
			octError("Making HUM-DEPENDENCY property");
			return;
		}
	}
	if(octFlushFacet(&CellInterface) != OCT_OK ) 
	{
		octError(cell_name);
		return;
	}

	fclose(input_file);
	fclose(term_file);
	fclose(down_delay_file);
	fclose(up_delay_file);
	return;
}



